;;;; cl-scopes/web/jwt - JWT creation and validation
;;;; inspired by: cljwt

(defpackage :scopes/web/jwt
  (:use :common-lisp)
  (:local-nicknames (:util :scopes/util)
                    (:jzon :com.inuoe.jzon))
  (:export #:create #:decode))

(in-package :scopes/web/jwt)
 
(defvar *header* 
  (util:to-b64 "{\"alg\":\"HS256\",\"typ\":\"JWT\"}" :scheme :uri))

(defvar *payload-format* "{\"sub\":~s,\"name\":~s,\"iat\":~s}")

(defun create (secret name &key (subject "scopes") (ttl 86400))
  (let* ((iat (util:to-unix-time (+ (get-universal-time) ttl)))
         (payload (util:to-b64 
                    (format nil *payload-format* 
                            (util:keyword-to-string subject)
                            (util:keyword-to-string name) iat)
                    :scheme :uri))
         (data (str:join "." (list *header* payload)))
         (sign (util:sign data secret)))
    (str:join "." (list data sign))))

(defun decode (token secret)
  (let (payload errors parts)
    (destructuring-bind (data &optional sig) (str:rsplit "." token :limit 2)
      (unless sig
        (return-from decode (values nil :malformed-token nil)))
      (unless (equal sig (util:sign data secret))
        (return-from decode (values nil :invalid-signature (list data sig))))
      (destructuring-bind (hjson &optional pjson) (str:split "." data)
        (setf payload (jzon:parse (util:from-b64 pjson)))
        (values payload errors parts)))))
